#ifndef REFLECTEDBGMAKER_H
#define REFLECTEDBGMAKER_H

#include <utilities/ConfigHandler.hh>

#include "BgMaker.hh"
#include "BgMaps.hh"


namespace Background {

  /**
   * \brief Estimate the background from off regions that
   * are reflected versions of the on region wrt. the run pointing position.
   *
   * \author The Heidelberg team
   */
  class ReflectedBgMaker : public BgMaker {
    
  public:
    
    ReflectedBgMaker(std::string name, 
		     Utilities::ConfigHandler& cfghandler,
		     std::string input = "Select", 
		     bool init_now = true);

    ReflectedBgMaker(Utilities::ConfigHandler& cfghandler,
		     std::string input = "Select", 
		     bool init_now = true);

    virtual ~ReflectedBgMaker();

    virtual void  BeginRun();
    virtual void  EndRun();
    
    int  GetNumberOffRegions()         { return fNumberOffRegions;};
    void SetNumberOffRegions(int noff) { fNumberOffRegions = noff;};
    void SetOffDistanceMax (double dist) {if (dist>0.) fMaxOffRegionDist = dist;} 
    void SetMinOnOffDistance(double dist) {if( dist>0. ) fMinOnOffDist = dist; }

    bool IsOnMapEvent();
    bool IsOffMapEvent();
    void FillOnExposure();
    void FillOffExposure();

    virtual bool CanMakeMaps(){return true;}
    virtual bool CanMakeStats(){return true;}

  private:
    void ConstructMaker(Utilities::ConfigHandler& cfghandler);

    static Tools::Circle* IsCircle(const Tools::Region* region);
    bool CreateOffRegions(const Tools::Region* onregion,
			  const Stash::Coordinate &obspos);
    bool CreateOffRegionsForOnCircle(const Tools::Circle* onregion,
				     const Stash::Coordinate &obspos);

    int    fMaxNumberOffRegions; /// maximum number of OFF regions
    int    fMinNumberOffRegions; /// minimum number of OFF regions
    double fMinOnOffDist;        /// minimum distance between ON and OFF region
    int    fNumberOffRegions;    /// actual number of OFF regions
    double fRotationAngleDeg;    /// rotation angle of initial OFF position

    double fCurrentOffset;
    double fMaxOffRegionDist;

    bool    fUseCircularOff;     /// use circular OFF regions instead of ON-shaped

    ClassDef( Background::ReflectedBgMaker, 3 );
  };
}



#endif

